@@ -1,6 +1,7 @@
 ;;; muse-latex.el --- publish entries in LaTex or PDF format
 
-;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+;;   Free Software Foundation, Inc.
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
 
@@ -37,6 +38,9 @@
 ;; Jim Ottaway (j DOT ottaway AT lse DOT ac DOT uk) implemented slides
 ;; and lecture notes.
 
+;; Karl Berry (karl AT freefriends DOT org) suggested how to escape
+;; additional special characters in image filenames.
+
 ;;; Code:
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -61,12 +65,19 @@
   :type 'string
   :group 'muse-latex)
 
+(defcustom muse-latex-pdf-browser "open %s"
+  "The program to use when browsing a published PDF file.
+This should be a format string."
+  :type 'string
+  :group 'muse-latex)
+
 (defcustom muse-latex-pdf-program "pdflatex"
   "The program that is called to generate PDF content from LaTeX content."
   :type 'string
   :group 'muse-latex)
 
-(defcustom muse-latex-pdf-cruft '(".aux" ".toc" ".out" ".log")
+(defcustom muse-latex-pdf-cruft
+  '(".aux" ".log" ".nav" ".out" ".snm" ".toc" ".vrb")
   "Extensions of files to remove after generating PDF output successfully."
   :type 'string
   :group 'muse-latex)
@@ -75,14 +86,24 @@
   "\\documentclass{article}
 
 \\usepackage[english]{babel}
-\\usepackage[latin1]{inputenc}
+\\usepackage{ucs}
+\\usepackage[utf8x]{inputenc}
 \\usepackage[T1]{fontenc}
 \\usepackage{hyperref}
 \\usepackage[pdftex]{graphicx}
 
+\\def\\museincludegraphics{%
+  \\begingroup
+  \\catcode`\\|=0
+  \\catcode`\\\\=12
+  \\catcode`\\#=12
+  \\includegraphics[width=0.75\\textwidth]
+}
+
 \\begin{document}
 
-\\title{<lisp>(muse-publishing-directive \"title\")</lisp>}
+\\title{<lisp>(muse-publish-escape-specials-in-string
+  (muse-publishing-directive \"title\") 'document)</lisp>}
 \\author{<lisp>(muse-publishing-directive \"author\")</lisp>}
 \\date{<lisp>(muse-publishing-directive \"date\")</lisp>}
 
@@ -112,7 +133,8 @@
 \\begin{document}
 \\begin{CJK*}<lisp>(muse-latexcjk-encoding)</lisp>
 
-\\title{<lisp>(muse-publishing-directive \"title\")</lisp>}
+\\title{<lisp>(muse-publish-escape-specials-in-string
+  (muse-publishing-directive \"title\") 'document)</lisp>}
 \\author{<lisp>(muse-publishing-directive \"author\")</lisp>}
 \\date{<lisp>(muse-publishing-directive \"date\")</lisp>}
 
@@ -138,20 +160,30 @@ filename."
   "\\documentclass[ignorenonframetext]{beamer}
 
 \\usepackage[english]{babel}
-\\usepackage[latin1]{inputenc}
+\\usepackage{ucs}
+\\usepackage[utf8x]{inputenc}
 \\usepackage[T1]{fontenc}
 \\usepackage{hyperref}
 
-\\begin{document}
+\\def\\museincludegraphics{%
+  \\begingroup
+  \\catcode`\\|=0
+  \\catcode`\\\\=12
+  \\catcode`\\#=12
+  \\includegraphics[width=0.50\\textwidth]
+}
 
-\\title{<lisp>(muse-publishing-directive \"title\")</lisp>}
+\\title{<lisp>(muse-publish-escape-specials-in-string
+  (muse-publishing-directive \"title\") 'document)</lisp>}
 \\author{<lisp>(muse-publishing-directive \"author\")</lisp>}
 \\date{<lisp>(muse-publishing-directive \"date\")</lisp>}
 
-\\maketitle
+\\begin{document}
+
+\\frame{\\titlepage}
 
 <lisp>(and muse-publish-generate-contents
-           \"\\\\tableofcontents\n\\\\newpage\")</lisp>\n\n"
+           \"\\\\frame{\\\\tableofcontents}\")</lisp>\n\n"
   "Header for publishing of slides using LaTeX.
 This may be text or a filename.
 
@@ -164,20 +196,31 @@ You must have the Beamer extension for LaTeX installed for this to work."
 \\usepackage{beamerarticle}
 
 \\usepackage[english]{babel}
-\\usepackage[latin1]{inputenc}
+\\usepackage{ucs}
+\\usepackage[utf8x]{inputenc}
 \\usepackage[T1]{fontenc}
 \\usepackage{hyperref}
+\\usepackage[pdftex]{graphicx}
 
-\\begin{document}
+\\def\\museincludegraphics{%
+  \\begingroup
+  \\catcode`\\|=0
+  \\catcode`\\\\=12
+  \\catcode`\\#=12
+  \\includegraphics[width=0.50\\textwidth]
+}
 
-\\title{<lisp>(muse-publishing-directive \"title\")</lisp>}
+\\title{<lisp>(muse-publish-escape-specials-in-string
+  (muse-publishing-directive \"title\") 'document)</lisp>}
 \\author{<lisp>(muse-publishing-directive \"author\")</lisp>}
 \\date{<lisp>(muse-publishing-directive \"date\")</lisp>}
 
-\\maketitle
+\\begin{document}
+
+\\frame{\\titlepage}
 
 <lisp>(and muse-publish-generate-contents
-           \"\\\\tableofcontents\n\\\\newpage\")</lisp>\n\n"
+           \"\\\\frame{\\\\tableofcontents}\")</lisp>\n\n"
   "Header for publishing of lecture notes using LaTeX.
 This may be text or a filename.
 
@@ -212,14 +255,14 @@ For more on the structure of this list, see
 
 (defcustom muse-latex-markup-strings
   '((image-with-desc . "\\begin{figure}[h]
-\\centering\\includegraphics[width=0.75\\textwidth]{%s.%s}
+\\centering\\museincludegraphics{%s.%s}|endgroup
 \\caption{%s}
 \\end{figure}")
     (image           . "\\begin{figure}[h]
-\\centering\\includegraphics[width=0.75\\textwidth]{%s.%s}
+\\centering\\museincludegraphics{%s.%s}|endgroup
 \\end{figure}")
     (image-link      . "%% %s
-\\includegraphics[width=0.75\\textwidth]{%s.%s}")
+\\museincludegraphics{%s.%s}|endgroup")
     (anchor-ref      . "\\ref{%s}")
     (url             . "\\url{%s}")
     (url-and-desc    . "\\href{%s}{%s}\\footnote{%1%}")
@@ -231,6 +274,7 @@ For more on the structure of this list, see
     (comment-begin   . "% ")
     (rule            . "\\vspace{.5cm}\\hrule\\vspace{.5cm}")
     (no-break-space  . "~")
+    (line-break      . "\\\\")
     (enddots         . "\\ldots{}")
     (dots            . "\\dots{}")
     (part            . "\\part{")
@@ -362,6 +406,7 @@ need to be escaped."
 
 (defcustom muse-latex-markup-specials-literal
   '((?\n . "\\\n")
+    (?\\ . "\\textbackslash{}")
     (?_  . "\\textunderscore{}")
     (?\< . "\\textless{}")
     (?\> . "\\textgreater{}")
@@ -395,7 +440,7 @@ These are applied to URLs."
   :group 'muse-latex)
 
 (defcustom muse-latex-markup-specials-image
-  '((?\\ . "\\textbackslash{}")     ; cannot find suitable replacement
+  '((?\\ . "\\\\")
     (?\< . "\\<")
     (?\> . "\\>")
     (?\$ . "\\$")
@@ -403,8 +448,8 @@ These are applied to URLs."
     (?\{ . "\\{")
     (?\} . "\\}")
     (?\& . "\\&")
-    (?\# . "\\#")                   ; cannot find suitable replacement
-    )
+    (?\# . "\\#")
+    (?\| . "\\|"))
   "A table of characters which must be represented specially.
 These are applied to image filenames."
   :type '(alist :key-type character :value-type string)
@@ -412,7 +457,8 @@ These are applied to image filenames."
 
 (defun muse-latex-decide-specials (context)
   "Determine the specials to escape, depending on CONTEXT."
-  (cond ((memq context '(underline emphasis document url-desc verbatim))
+  (cond ((memq context '(underline emphasis document url-desc verbatim
+                                   footnote))
          muse-latex-markup-specials-document)
         ((eq context 'image)
          muse-latex-markup-specials-image)
@@ -468,13 +514,14 @@ and it will do what you expect."
 This is used by the slides and lecture-notes publishing styles."
   (let ((title (cdr (assoc "title" attrs))))
     (goto-char beg)
-    (muse-insert-markup "\\begin{frame}\n")
+    (muse-insert-markup "\\begin{frame}[fragile]\n")
     (when title
       (muse-insert-markup "\\frametitle{")
       (insert title)
       (muse-insert-markup "}\n"))
-    (goto-char end)
-    (muse-insert-markup "\n\\end{frame}")))
+    (save-excursion
+      (goto-char end)
+      (muse-insert-markup "\n\\end{frame}"))))
 
 ;;; Post-publishing functions
 
@@ -540,7 +587,7 @@ default Muse will add a footnote for each link."
       "")))
 
 (defun muse-latex-pdf-browse-file (file)
-  (shell-command (concat "open " file)))
+  (shell-command (format muse-latex-pdf-browser file)))
 
 (defun muse-latex-pdf-generate (file output-path final-target)
   (apply
@@ -549,10 +596,11 @@ default Muse will add a footnote for each link."
    (function
     (lambda (file output-path)
       (let* ((fnd (file-name-directory output-path))
-             (command (format "cd \"%s\"; %s \"%s\""
-                              fnd muse-latex-pdf-program
+             (command (format "%s \"%s\""
+                              muse-latex-pdf-program
                               (file-relative-name file fnd)))
              (times 0)
+             (default-directory fnd)
              result)
         ;; XEmacs can sometimes return a non-number result.  We'll err
         ;; on the side of caution by continuing to attempt to generate
