@@ -1,6 +1,7 @@
 ;;; muse-mode.el --- mode for editing Muse files; has font-lock support
 
-;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
+;;   Free Software Foundation, Inc.
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
 
@@ -38,6 +39,9 @@
 ;; muse-next-reference and muse-previous-reference involving links
 ;; that begin at point 1.
 
+;; Gregory Collins (greg AT gregorycollins DOT net) fixed a bug with
+;; paragraph separation and headings when filling.
+
 ;;; Code:
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -227,11 +231,17 @@ index at intervals."
   ;; Make fill work nicely with item lists
   (let ((regexp (concat "\\s-+\\(-\\|[0-9]+\\.\\)\\s-+"
                         "\\|\\[[0-9]+\\]\\s-*"
-                        "\\|.*\\s-*::\\s-+")))
+                        "\\|.*\\s-*::\\s-+"
+                        "\\|\\*+\\s-+")))
     (set (make-local-variable 'adaptive-fill-regexp)
          (concat regexp "\\|\\s-*"))
     (set (make-local-variable 'paragraph-start)
-         (concat paragraph-start "\\|" regexp)))
+         (concat paragraph-start "\\|" regexp))
+    (set (make-local-variable 'paragraph-separate)
+         (concat paragraph-separate "\\|\\*+\\s-+")))
+  (set (make-local-variable 'fill-paragraph-function)
+       'muse-mode-fill-paragraph)
+
   ;; Comment syntax is `; comment'
   (set (make-local-variable 'comment-start)
        "; ")
@@ -273,15 +283,36 @@ fill mode."
            (string= (or (match-string 0) "")
                     "[[")))))
 
+(defun muse-mode-fill-paragraph (arg)
+  "If a definition list is at point, use special filling rules for it.
+Otherwise return nil to let the normal filling function take care
+of things.
+
+ARG is passed to `fill-paragraph'."
+  (let ((count 2))
+    (and (not (muse-mode-fill-nobreak-p))
+         (save-excursion
+           (beginning-of-line)
+           (and (looking-at muse-dl-term-regexp)
+                (prog1 t
+                  ;; Take initial whitespace into account
+                  (when (looking-at (concat "[" muse-regexp-blank "]+"))
+                    (setq count (+ count (length (match-string 0))))))))
+         (let ((fill-prefix (make-string count ?\ ))
+               (fill-paragraph-function nil))
+           (prog1 t
+             (fill-paragraph arg))))))
+
 (defun muse-mode-flyspell-p ()
   "Return non-nil if we should allow spell-checking to occur at point.
 Otherwise return nil.
 
 This is used to keep links from being improperly colorized by flyspell."
-  (and (not (get-text-property (if (bobp) (point) (1- (point)))
-                               'muse-link))
-       (save-match-data
-         (null (muse-link-at-point)))))
+  (let ((pos (if (bobp) (point) (1- (point)))))
+    (and (not (get-text-property pos 'muse-no-flyspell))
+         (not (get-text-property pos 'muse-link))
+         (save-match-data
+           (null (muse-link-at-point))))))
 
 ;;;###autoload
 (defun muse-mode-choose-mode ()
@@ -499,11 +530,17 @@ Valid values of OPERATION are 'increase and 'decrease."
    (muse-make-link (file-relative-name (read-file-name "Link: "))
                    (read-string "Text: "))))
 
+(defcustom muse-insert-url-initial-input "http://"
+  "The string to insert before reading a URL interactively.
+This is used by the `muse-insert-url' command."
+  :type 'string
+  :group 'muse-mode)
+
 (defun muse-insert-url ()
   "Insert a URL, with optional description, at point."
   (interactive)
   (insert
-   (muse-make-link (read-string "URL: ")
+   (muse-make-link (read-string "URL: " muse-insert-url-initial-input)
                    (read-string "Text: "))))
 
 ;;;###autoload
@@ -557,12 +594,15 @@ in `muse-project-alist'."
                   (find-file-other-window link)
                 (find-file link))))))
       (when anchor
-        (let ((pos (point)))
+        (let ((pos (point))
+              (regexp (concat "^\\W*" (regexp-quote anchor) "\\b"))
+              last)
           (goto-char (point-min))
-          (unless (re-search-forward (concat "^\\W*" (regexp-quote anchor)
-                                             "\\b")
-                                     nil t)
-            (goto-char pos)))))))
+          (while (and (setq last (re-search-forward regexp nil t))
+                      (muse-link-at-point)))
+          (unless last
+            (goto-char pos)
+            (message "Could not find anchor `%s'" anchor)))))))
 
 (defun muse-visit-link (link &optional other-window)
   "Visit the URL or link named by LINK."
@@ -580,18 +620,24 @@ in `muse-project-alist'."
                                             (cddr muse-current-project))
          current-prefix-arg))
   (setq style (muse-style style))
+  (muse-project-publish-this-file nil style)
   (let* ((output-dir (muse-style-element :path style))
-         (result-path (muse-publish-output-file buffer-file-name output-dir
+         (output-suffix (muse-style-element :osuffix style))
+         (output-path (muse-publish-output-file buffer-file-name output-dir
                                                 style))
+         (target (if output-suffix
+                     (concat (muse-path-sans-extension output-path)
+                             output-suffix)
+                   output-path))
          (muse-current-output-style (list :base (car style)
                                           :path output-dir)))
-    (if (not (file-readable-p result-path))
-        (error "Cannot open output file '%s'" result-path)
+    (if (not (file-readable-p target))
+        (error "Cannot open output file '%s'" target)
       (if other-window
-          (find-file-other-window result-path)
+          (find-file-other-window target)
         (let ((func (muse-style-element :browser style t)))
           (if func
-              (funcall func result-path)
+              (funcall func target)
             (message "The %s publishing style does not support browsing."
                      style)))))))
 
